module shpc_create_m

  implicit none

contains

  subroutine shpc_create(hshp, shpc_dir, cyclone)

    ! Libraries:
    use jumble, only: new_unit
    use shapelib, only: shpt_point, shpt_polygon, ftdouble, ftinteger
    use shapelib_03, only: shp_create_03, dbf_add_field_03

    use derived_types, only: shpc

    TYPE(shpc), intent(out):: hshp
    character(len = *), intent(in):: shpc_dir
    logical, intent(in):: cyclone

    ! Local:
    integer unit

    !---------------------------------------------------------------------

    call shp_create_03(shpc_dir // "/extremum", shpt_point, &
         hshp%extremum)
    call dbf_add_field_03(hshp%extr_ssh, hshp%extremum, 'ssh', ftdouble, &
         nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(hshp%extr_date, hshp%extremum, 'days_1950', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(hshp%extr_eddy_index, hshp%extremum, 'eddy_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(hshp%extr_interp, hshp%extremum, 'interpolat', &
         ftinteger, nwidth = 1, ndecimals = 0)
    call dbf_add_field_03(hshp%extr_valid, hshp%extremum, 'valid', &
         ftinteger, nwidth = 1, ndecimals = 0)
    call dbf_add_field_03(hshp%extr_speed, hshp%extremum, 'speed', ftdouble, &
         nwidth = 13, ndecimals = 6)

    call shp_create_03(shpc_dir // "/outermost_contour", shpt_polygon, &
         hshp%outermost)
    call dbf_add_field_03(hshp%out_r_eq_area, hshp%outermost, &
         'r_eq_area', ftdouble, nwidth = 10, ndecimals = 4)
    call dbf_add_field_03(hshp%out_ssh, hshp%outermost, 'ssh', ftdouble, &
         nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(hshp%out_date, hshp%outermost, 'days_1950', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(hshp%out_eddy_index, hshp%outermost, 'eddy_index', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(hshp%out_radius4, hshp%outermost, 'radius4', &
         ftinteger, nwidth = 2, ndecimals = 0)

    call shp_create_03(shpc_dir // "/max_speed_contour", shpt_polygon, &
         hshp%max_speed)
    call dbf_add_field_03(hshp%max_speed_r_eq_area, hshp%max_speed, &
         'r_eq_area', ftdouble, nwidth = 10, ndecimals = 4)
    call dbf_add_field_03(hshp%max_speed_ssh, hshp%max_speed, 'ssh', &
         ftdouble, nwidth = 13, ndecimals = 6)
    call dbf_add_field_03(hshp%max_speed_date, hshp%max_speed, 'days_1950', &
         ftinteger, nwidth = 5, ndecimals = 0)
    call dbf_add_field_03(hshp%max_speed_eddy_index, hshp%max_speed, &
         'eddy_index', ftinteger, nwidth = 5, ndecimals = 0)

    hshp%cyclone = cyclone
    call new_unit(unit)
    open(unit, file  = shpc_dir // "/orientation.txt", status = "replace", &
         action  = "write")

    if (cyclone) then
       write(unit, fmt = *) "cyclones"
    else
       write(unit, fmt = *) "anticyclones"
    end if
    
    close(unit)

  end subroutine shpc_create

end module shpc_create_m
