#!/usr/bin/env python3

import plot_eddy_contours
from matplotlib import animation

def bbox_union(b1, b2):
    if b1 is None:
        return b2
    else:
        return [min(b1[0], b2[0]), min(b1[1], b2[1]), max(b1[2], b2[2]),
                max(b1[3], b2[3])]

def compute_bbox(ishape_lists, reader_outer):
    bbox = None

    for ishape_list in ishape_lists.values():
        for ishape in ishape_list:
            try:
                bbox = bbox_union(bbox, reader_outer.shape(ishape).bbox)
            except AttributeError:
                pass

    return bbox

def func(d, ax, ishape_lists_all, readers_all, bbox, light):
    """To be passed as argument to the animation
    function. ishape_lists_all and readers_all are lists of
    dictionaries.

    """
    
    ax.cla()
    ax.set_xlim(bbox[0], bbox[2])
    ax.set_ylim(bbox[1], bbox[3])
    
    for ishape_lists, readers in zip(ishape_lists_all, readers_all):
        plot_eddy_contours.snapshot(ax, ishape_lists[d], readers, light = light)
        
    ax.gridlines(draw_labels = True) 
    ax.coastlines()
    ax.set_title(f"d = {d}", y = 1.05)

def anim_eddies(fig, ax, d_init, open_shpc_return, window, d_min, d_max, light):
    """open_shpc_return is a list of tuples."""
    
    ishape_lists_all = []
    readers_all = []
    
    for readers, ishape_last in open_shpc_return:
        ishape_lists = {d:
                        plot_eddy_contours.select_ishapes(d, d_init,
                                                          ishape_last,
                                                          readers["extremum"],
                                                          window)
                        for d in range(d_min, d_max + 1)}
        ishape_lists_all.append(ishape_lists)
        readers_all.append(readers)

    if window is None:
        bbox = None
        
        for readers, ishape_lists in zip(readers_all, ishape_lists_all):
            b2 = compute_bbox(ishape_lists, readers["outermost_contour"])
            bbox = bbox_union(bbox, b2)
    else:
        bbox = window

    ani = animation.FuncAnimation(fig, func, range(d_min, d_max + 1),
                                  fargs = (ax, ishape_lists_all, readers_all,
                                           bbox, light),
                                  interval = 500)
    return ani

if __name__ == "__main__":
    import argparse
    import sys
    import matplotlib.pyplot as plt
    import cartopy.crs as ccrs
    import pygifsicle
    from os import path
    sys.path.append(path.join(sys.path[0], "../../Common"))
    import util_eddies

    # Process arguments:
    parser = argparse.ArgumentParser()
    parser.add_argument("shpc_dir", help = "directories containing the "
                        "collections of shapefiles", nargs = "+")
    parser.add_argument("-d", "--dates", type = int, nargs = 2,
                        metavar = ("d_min", "d_max"))
    parser.add_argument("-l", "--light", help = "lighter plot",
                        action = "store_true")
    parser.add_argument("-w", "--window", help = "choose a limited plot window",
                        type = float, nargs = 4,
                        metavar = ("llcrnrlon", "llcrnrlat", "urcrnrlon",
                                   "urcrnrlat"))
    args = parser.parse_args()
    #--

    readers, d_init, ishape_last = util_eddies.open_shpc(args.shpc_dir[0])

    # Do some checks on first input SHPC:

    if ishape_last is None: sys.exit("Missing ishape_last.txt")

    if args.dates:
        d_min = args.dates[0]
        d_max = args.dates[1]
        
        if not d_init <= d_min < d_max <= d_init + len(ishape_last) - 1:
            sys.exit("Bad dates specified")
    else:
        d_min = d_init
        d_max = d_init + len(ishape_last) - 1
    #--
    
    open_shpc_return = [(readers, ishape_last)]
    
    # We assume (and do not check) that ishape_last.txt exists and the
    # dates are the same in all other SHPC:
    for shpc_dir in args.shpc_dir[1:]:
        readers, d_init, ishape_last = util_eddies.open_shpc(shpc_dir)
        open_shpc_return.append((readers, ishape_last))
        
    fig = plt.figure()
    projection = ccrs.PlateCarree()
    ax = plt.axes(projection = projection)
    ani = anim_eddies(fig, ax, d_init, open_shpc_return, args.window, d_min,
                      d_max, args.light)
    ani.save("eddy_contours.gif", writer  = "imagemagick")

    pygifsicle.gifsicle("eddy_contours.gif",  options = ["--no-loopcount"])
    # (The repeat = False option of the save method of
    # FuncAnimation does not work.)

    print('Created file "eddy_contours.gif".')
