module write_aux_m

  implicit none

contains

  subroutine write_aux(corner_deg, step_deg, nlon, nlat, n_shapes, shpc_dir)

    ! Write auxiliary files "grid_nml.txt" and "ishape_last.txt".

    ! Libraries:
    use jumble, only: new_unit

    ! Note: we do not use assumed-shape arrays here for corner_deg and
    ! step_deg because of an internal compiler error of ifort
    ! 19.0. The bug seems corrected in ifort 19.1. Assumed-shape
    ! arrays are allowed in a namelist in Fortran 2003.

    real, intent(in):: corner_deg(2)
    ! longitude and latitude of the corner of the whole grid, in degrees

    real, intent(in):: step_deg(2)
    ! longitude and latitude steps, in degrees

    integer, intent(in):: nlon, nlat
    ! size of ssh array in input NetCDF, assuming no repeated point if
    ! the grid is global

    integer, intent(in):: n_shapes
    character(len = *), intent(in):: shpc_dir

    ! Local:
    namelist /grid_nml/ corner_deg, step_deg, nlon, nlat
    integer unit

    !--------------------------------------------------------------------

    call new_unit(unit)
    open(unit, file = shpc_dir // "/grid_nml.txt", status = "replace", &
         action = "write")
    write(unit, nml = grid_nml)
    close(unit)

    open(unit, file = shpc_dir // "/ishape_last.txt", status = "replace", &
         action = "write")
    write(unit, fmt = *) n_shapes - 1
    close(unit)

  end subroutine write_aux
  
end module write_aux_m
